// Run this as follows: go run naif.go 50000    [where 50000 is the size of the randomly generated slice]

package main

import (
	"fmt"
	"math/rand"
	"os"
	"sort"
	"strconv"
	"time"
)

func gen(n int) []int {
	res := make([]int, n)
	for i := 0; i < n; i++ {
		res[i] = rand.Intn(1000)
	}
	return res
}

func selectionSort(x []int) {
	var i, j int
	n := len(x)
	for i = 0; i < n; i++ {
		imin := i
		min := x[i]
		for j = i + 1; j < n; j++ {
			if x[j] < min {
				imin = j
				min = x[j]
			}
		}
		x[imin], x[i] = x[i], x[imin]
	}
}


func insertionSort(x []int) {
	var i, j int
	n := len(x)
	for i = 1; i < n; i++ {
		for j = 0; j < i; j++ {
			if x[j] > x[i] {
				break
			}
		}
		// insert x[i] in position j
		t := x[i]
		for k := i - 1; k >= j; k-- {
			x[k + 1] = x[k]
		}
		x[j] = t
	}
}

func bubbleSort(x []int) {
	var i, j int
	n := len(x)
	for i = n; i > 0; i-- {
		for j = 0; j < i - 1; j++ {
			if x[j] > x[j + 1] {
				x[j], x[j + 1] = x[j + 1], x[j]
			}
		}
	}
}

func trySort(x []int, sortFunction func ([]int)) int {
	// Copy slice
	y := make([]int, len(x))
	copy(y, x)
	// Time
	start := time.Now().UnixNano()
	sortFunction(y)
	end := time.Now().UnixNano()
	if !sort.IntsAreSorted(y) {
		fmt.Printf("ERRORE!")
	}
	return int(end - start)
}

func main() {
	n, _ := strconv.Atoi(os.Args[1])	
	x := gen(n)
	fmt.Printf("Selection sort:   \t%15d ns.\n", trySort(x, selectionSort))
	fmt.Printf("Insertion sort:   \t%15d ns.\n", trySort(x, insertionSort))
	fmt.Printf("Bubblesort:       \t%15d ns.\n", trySort(x, bubbleSort))
	fmt.Printf("Golang sort.Ints: \t%15d ns.\n", trySort(x, sort.Ints))
}

